@@ -1,9 +1,11 @@
+from collections import defaultdict
 import boto3
 import pendulum
 import pandas as pd
 from logging import getLogger
 from botocore.exceptions import ProfileNotFound, ClientError
 from sumatra.config import CONFIG
+from typing import Dict
 
 logger = getLogger("sumatra.admin")
 
@@ -81,22 +83,27 @@ class AdminClient:
             self._remove_tenant(username, current)
         self._add_tenant(username, tenant)
 
-    def _get_keys_with_prefix(self, prefix):
-        keys = {}
-        resp = self._apigateway.get_api_keys()
+    def _get_keys(self) -> Dict:
+        keys = defaultdict(dict)
+        resp = self._apigateway.get_api_keys(includeValues=True)
         for item in resp.get("items", []):
             name = item["name"]
-            if name.startswith(prefix):
-                keys[name[len(prefix) :]] = item["id"]
-        return keys
+            if name.startswith("api_") or name.startswith("sdk_"):
+                keys[name[4:]][name[:3]] = {"id": item["id"], "value": item["value"]}
+        return dict(keys)
 
-    def get_api_keys(self):
-        return self._get_keys_with_prefix("api_")
+    def get_keys(self) -> pd.DataFrame:
+        rows = []
+        for tenant, keys in self._get_keys().items():
+            for typ, val in keys.items():
+                rows.append((tenant, typ, val["value"]))
+        return (
+            pd.DataFrame(rows, columns=("Tenant", "Type", "Key"))
+            .set_index(["Tenant", "Type"])
+            .sort_index()
+        )
 
-    def get_sdk_keys(self):
-        return self._get_keys_with_prefix("sdk_")
-
-    def get_usage_plans(self):
+    def _get_usage_plans(self):
         ids = {}
         resp = self._apigateway.get_usage_plans()
         for item in resp.get("items", []):
@@ -104,41 +111,35 @@ class AdminClient:
             ids[name] = item["id"]
         return ids
 
-    def _get_usage_by_plan(self, days):
+    def _get_key_usage(self, days):
         if not days:
             raise Exception("empty date range")
-        data_by_plan = {}
-        for name, usage_plan_id in self.get_usage_plans().items():
+        usage_by_key = {}
+        for usage_plan_id in self._get_usage_plans().values():
             resp = self._apigateway.get_usage(
                 usagePlanId=usage_plan_id, startDate=days[0], endDate=days[-1]
             )
-            data = {}
             for key, usage in resp.get("items", {}).items():
                 if len(usage) != len(days):
                     raise Exception(
                         f"expected {len(days)} data points, found {len(usage)}"
                     )
                 used = [row[0] for row in usage]
-                data[key] = used
-            data_by_plan[name] = data
-        return data_by_plan
+                usage_by_key[key] = used
+        return usage_by_key
 
-    def _usage_for_key(self, data_by_plan, key):
-        for _, usage_data in data_by_plan.items():
-            for plan_key, usage in usage_data.items():
-                if plan_key == key:
-                    return usage
-        return 0
-
-    def usage_report(self, start_date=None, end_date=None):
+    def get_key_usage(self, start_date=None, end_date=None):
         end_date = end_date or pendulum.today()
         start_date = start_date or end_date.subtract(days=6)
         days = [d.to_date_string() for d in end_date - start_date]
-        data_by_plan = self._get_usage_by_plan(days)
-        api_keys = self.get_api_keys()
-        sdk_keys = self.get_sdk_keys()
-        df = pd.DataFrame(index=days)
-        for tenant in self.list_tenants():
-            for key_type, key in [("api", api_keys[tenant]), ("sdk", sdk_keys[tenant])]:
-                df[tenant + "_" + key_type] = self._usage_for_key(data_by_plan, key)
-        return df.transpose()
+        usage = self._get_key_usage(days)
+        rows = []
+        for tenant, keys in self._get_keys().items():
+            for typ, val in keys.items():
+                used = usage.get(val["id"], [0 for _ in days])
+                rows.append([tenant, typ] + used)
+        return (
+            pd.DataFrame(rows, columns=["Tenant", "Type"] + days)
+            .set_index(["Tenant", "Type"])
+            .sort_index()
+        )
