@@ -170,9 +170,7 @@ def timeline_show(args):
 
 
 def timeline_upload(args):
-    with open(args.jsonl, "r") as f:
-        jsonl = f.read()
-        Client().create_timeline_from_jsonl(jsonl, args.timeline)
+    Client().create_timeline_from_file(args.file, args.timeline)
     print(f"Successfully uploaded to timeline '{args.timeline}'.")
 
 
@@ -225,6 +223,14 @@ def admin_list_users(args):
     print("\n".join(users))
 
 
+def admin_show_keys(args):
+    print(AdminClient(args.aws_profile).get_keys())
+
+
+def admin_show_key_usage(args):
+    print(AdminClient(args.aws_profile).get_key_usage())
+
+
 ADMIN_COMMANDS = [
     {
         "name": "list-tenants",
@@ -246,6 +252,16 @@ ADMIN_COMMANDS = [
         "help": "list all users",
         "handler": admin_list_users,
     },
+    {
+        "name": "show-keys",
+        "help": "show api and sdk keys",
+        "handler": admin_show_keys,
+    },
+    {
+        "name": "show-key-usage",
+        "help": "show api and sdk key daily usage",
+        "handler": admin_show_key_usage,
+    },
 ]
 
 TIMELINE_COMMANDS = [
@@ -418,10 +434,8 @@ def main():
                         )
                     if subcmd["name"] == "upload":
                         p.add_argument(
-                            "--jsonl",
-                            metavar="FILE",
-                            help="file with event data in JSON Lines format",
-                            required=True,
+                            "file",
+                            help="file with event data in JSON Lines format (may be gzipped)",
                         )
                 if cmd["name"] == "admin":
                     if subcmd["name"] in ["current-tenant", "assign-tenant"]:
