@@ -5,6 +5,7 @@ import time
 import logging
 import awswrangler as wr
 import boto3
+import gzip
 import python_graphql_client
 import pandas as pd
 import requests
@@ -45,6 +46,17 @@ def _load_scowl_files(dir: str) -> Dict[str, str]:
     return scowls
 
 
+def _splitext(path: str):
+    fullext = ""
+    while True:
+        path, ext = os.path.splitext(path)
+        if ext:
+            fullext = ext + fullext
+        else:
+            break
+    return os.path.basename(path), fullext
+
+
 class Client:
     """
     Client to connect to Sumatra GraphQL API
@@ -199,9 +211,7 @@ class Client:
         if "errors" in ret:
             raise Exception(f"Error: {ret['errors'][0]['message']}")
 
-    def create_branch_from_scowl(
-        self, scowl: str, branch: Optional[str] = None
-    ) -> str:
+    def create_branch_from_scowl(self, scowl: str, branch: Optional[str] = None) -> str:
         """
         Create (or overwrite) branch with single file of scowl source code.
 
@@ -226,7 +236,7 @@ class Client:
         if "error" in b:
             raise Exception(b["error"])
 
-        return b['name']
+        return b["name"]
 
     def create_branch_from_dir(
         self, scowl_dir: Optional[str] = None, branch: Optional[str] = None
@@ -263,7 +273,7 @@ class Client:
         if "error" in b:
             raise Exception(b["error"])
 
-        return b['name']
+        return b["name"]
 
     def publish_dir(self, scowl_dir: Optional[str] = None) -> None:
         """
@@ -274,7 +284,7 @@ class Client:
         """
         scowl_dir = scowl_dir or CONFIG.scowl_dir
         logger.info(f"Publishing dir '{scowl_dir}' to LIVE.")
-        branch = self.create_branch_from_dir(scowl_dir)
+        branch = self.create_branch_from_dir(scowl_dir, "main")
         self.publish_branch(branch)
 
     def publish_branch(self, branch: Optional[str] = None) -> None:
@@ -308,7 +318,7 @@ class Client:
             scowl: Scowl source code as string.
         """
         logger.info("Publishing scowl to LIVE.")
-        branch = self.create_branch_from_scowl(scowl)
+        branch = self.create_branch_from_scowl(scowl, "main")
         self.publish_branch(branch)
 
     def diff_branch_with_live(
@@ -592,6 +602,17 @@ class Client:
             jsonl: JSON event data, one JSON dict per line.
             timeline: Timeline name.
         """
+
+        if not jsonl.endswith("\n"):
+            jsonl += "\n"
+        data = gzip.compress(bytes(jsonl, "utf-8"))
+        self._create_timeline_from_jsonl_gz(data, timeline)
+
+    def _create_timeline_from_jsonl_gz(
+        self,
+        data: bytes,
+        timeline: str,
+    ) -> None:
         query = """
             mutation SaveTimelineMutation($id: String!,
                                           $filename: String!) {
@@ -602,16 +623,14 @@ class Client:
         """
 
         ret = self._gql_client.execute(
-            query=query, variables={"id": timeline, "filename": "sdk.jsonl"}
+            query=query, variables={"id": timeline, "filename": "timeline.jsonl.gz"}
         )
         if "errors" in ret:
             raise Exception(ret["errors"][0]["message"])
 
         url = ret["data"]["saveTimeline"]["uploadUrl"]
 
-        if not jsonl.endswith("\n"):
-            jsonl += "\n"
-        http_response = requests.put(url, data=jsonl)
+        http_response = requests.put(url, data=data)
         if http_response.status_code != 200:
             raise Exception(http_response.error)
 
@@ -643,6 +662,29 @@ class Client:
             f"Timeline processing timed out after {retries * delay} seconds."
         )
 
+    def create_timeline_from_file(self, filename: str, timeline: str) -> None:
+        """
+        Create (or overwrite) timeline from events stored in a file.
+
+        Supported file types: `.jsonl`, `.jsonl.gz`
+
+        Arguments:
+            filename: Name of events file to upload.
+            timeline: Timeline name.
+        """
+
+        _, ext = _splitext(filename)
+
+        if ext in (".jsonl.gz", ".json.gz"):
+            with open(filename, "rb") as f:
+                self._create_timeline_from_jsonl_gz(f.read(), timeline)
+        elif ext in (".jsonl", ".json"):
+            with open(filename, "r") as f:
+                jsonl = f.read()
+                self.create_timeline_from_jsonl(jsonl, timeline)
+        else:
+            raise Exception(f"Unsupported file extension: {ext}")
+
     def _df_to_jsonl(self, df, event_type, timestamp_column=None):
         df = df.copy()
         df["_type"] = event_type
@@ -867,15 +909,30 @@ class Materialization:
         """
         Timelines materialized by the job.
         """
-        self._wait_for_processing()
+        if self._mtr is None:
+            self._mtr = self._get_materialization()
         return self._mtr["timelines"]
 
+    @property
+    def event_types(self) -> List[str]:
+        """
+        Union of all event types from timelines and topology
+        """
+        timelines = self._client.get_timelines()
+        event_types = set()
+        for timeline in self.timelines:
+            tl = timelines.loc[timeline]
+            event_types.update(tl.event_types)
+        event_types.update(self._client.get_branch(self.branch)["event_types"])
+        return list(event_types)
+
     @property
     def branch(self) -> str:
         """
         Topology branch used for materialization.
         """
-        self._wait_for_processing()
+        if self._mtr is None:
+            self._mtr = self._get_materialization()
         return self._mtr["branch"]
 
     @property
@@ -883,7 +940,8 @@ class Materialization:
         """
         Unique hash identifying the job.
         """
-        self._wait_for_processing()
+        if self._mtr is None:
+            self._mtr = self._get_materialization()
         return self._mtr["hash"]
 
     @property
@@ -894,7 +952,7 @@ class Materialization:
         self._wait_for_processing()
         return self._mtr["path"]
 
-    def get_events(self, event_type, features=[]) -> pd.DataFrame:
+    def get_events(self, event_type: str, features: List[str] = []) -> pd.DataFrame:
         """
         Return enriched events of specified type. Waits if
         job is still processing.
@@ -906,6 +964,9 @@ class Materialization:
         Returns:
             Enriched events
         """
+        if event_type not in self.event_types:
+            raise Exception(f"Event type '{event_type}' not found.")
+
         self._wait_for_processing()
         session = self._client._get_session()
 
@@ -941,6 +1002,9 @@ class Materialization:
             Event-level errors
         """
 
+        if event_type not in self.event_types:
+            raise Exception(f"Event type '{event_type}' not found.")
+
         self._wait_for_processing()
         session = self._client._get_session()
 
